@@ -856,7 +856,7 @@ bgp_parse_capabilities(struct bgp_conn *conn, byte *opt, int len)
 	    conn->advertised_as = get_u32(opt + 2);
 	  break;
 
-	case 69: /* ADD-PATH capability, draft */
+	case 69: /* ADD-PATH capability, RFC 7911 */
 	  if (cl % 4)
 	    goto err;
 	  for (i = 0; i < cl; i += 4)
@@ -1037,8 +1037,13 @@ bgp_rx_open(struct bgp_conn *conn, byte *pkt, uint len)
   p->gr_ready = p->cf->gr_mode && conn->peer_gr_able;
   p->ext_messages = p->cf->enable_extended_messages && conn->peer_ext_messages_support;
 
+  /* Update RA mode */
   if (p->add_path_tx)
     p->p.accept_ra_types = RA_ANY;
+  else if (p->cf->secondary)
+    p->p.accept_ra_types = RA_ACCEPTED;
+  else
+    p->p.accept_ra_types = RA_OPTIMAL;
 
   DBG("BGP: Hold timer set to %d, keepalive to %d, AS to %d, ID to %x, AS4 session to %d\n", conn->hold_time, conn->keepalive_time, p->remote_as, p->remote_id, p->as4_session);
 
@@ -1489,38 +1494,72 @@ bgp_error_dsc(unsigned code, unsigned subcode)
   return buff;
 }
 
+/* RFC 8203 - shutdown communication message */
+static int
+bgp_handle_message(struct bgp_proto *p, byte *data, uint len, byte **bp)
+{
+  byte *msg = data + 1;
+  uint msg_len = data[0];
+  uint i;
+
+  /* Handle zero length message */
+  if (msg_len == 0)
+    return 1;
+
+  /* Handle proper message */
+  if ((msg_len > 128) && (msg_len + 1 > len))
+    return 0;
+
+  /* Some elementary cleanup */
+  for (i = 0; i < msg_len; i++)
+    if (msg[i] < ' ')
+      msg[i] = ' ';
+
+  proto_set_message(&p->p, msg, msg_len);
+  *bp += bsprintf(*bp, ": \"%s\"", p->p.message);
+  return 1;
+}
+
 void
 bgp_log_error(struct bgp_proto *p, u8 class, char *msg, unsigned code, unsigned subcode, byte *data, unsigned len)
 {
-  const byte *name;
-  byte *t, argbuf[36];
+  byte argbuf[256], *t = argbuf;
   unsigned i;
 
   /* Don't report Cease messages generated by myself */
   if (code == 6 && class == BE_BGP_TX)
     return;
 
-  name = bgp_error_dsc(code, subcode);
-  t = argbuf;
+  /* Reset shutdown message */
+  if ((code == 6) && ((subcode == 2) || (subcode == 4)))
+    proto_set_message(&p->p, NULL, 0);
+
   if (len)
     {
-      *t++ = ':';
-      *t++ = ' ';
-
+      /* Bad peer AS - we would like to print the AS */
       if ((code == 2) && (subcode == 2) && ((len == 2) || (len == 4)))
 	{
-	  /* Bad peer AS - we would like to print the AS */
-	  t += bsprintf(t, "%d", (len == 2) ? get_u16(data) : get_u32(data));
+	  t += bsprintf(t, ": %u", (len == 2) ? get_u16(data) : get_u32(data));
 	  goto done;
 	}
+
+      /* RFC 8203 - shutdown communication */
+      if (((code == 6) && ((subcode == 2) || (subcode == 4))))
+	if (bgp_handle_message(p, data, len, &t))
+	  goto done;
+
+      *t++ = ':';
+      *t++ = ' ';
       if (len > 16)
 	len = 16;
       for (i=0; i<len; i++)
 	t += bsprintf(t, "%02x", data[i]);
     }
- done:
+
+done:
   *t = 0;
-  log(L_REMOTE "%s: %s: %s%s", p->p.name, msg, name, argbuf);
+  const byte *dsc = bgp_error_dsc(code, subcode);
+  log(L_REMOTE "%s: %s: %s%s", p->p.name, msg, dsc, argbuf);
 }
 
 static void
@@ -1566,7 +1605,17 @@ bgp_rx_notification(struct bgp_conn *conn, byte *pkt, uint len)
   if (err) 
     {
       bgp_update_startup_delay(p);
-      bgp_stop(p, 0);
+      bgp_stop(p, 0, NULL, 0);
+    }
+  else
+    {
+      uint subcode_bit = 1 << ((subcode <= 8) ? subcode : 0);
+      if (p->cf->disable_after_cease & subcode_bit)
+      {
+	log(L_INFO "%s: Disabled after Cease notification", p->p.name);
+	p->startup_delay = 0;
+	p->p.disabled = 1;
+      }
     }
 }
 
