@@ -498,7 +498,8 @@ static_same_dest(struct static_route *x, struct static_route *y)
 static inline int
 static_same_rte(struct static_route *x, struct static_route *y)
 {
-  return static_same_dest(x, y) && i_same(x->cmds, y->cmds);
+  /* Note that i_same() requires arguments in (new, old) order */
+  return static_same_dest(x, y) && i_same(y->cmds, x->cmds);
 }
 
 
